prism.run([function () {

    // check service availability (not exist in admin app)
    if (prism.$injector.has('table.services.dataService')) {

        var $dataService = prism.$injector.get('table.services.dataService');
        var $query = prism.$injector.get('dashboard-base.services.$query');

        // override function in order to apply logic on the last page of the table.
        $dataService.getData = function (widget, offset, limit, ungroup, callback) {

            var query = $query.buildWidgetQuery(widget);

            query.ungroup = ungroup;
            query.count = parseInt(limit, 10);
            query.offset = parseInt(offset, 10);

            $query.jaql(query).then(function (response) {
                //console.info('response: offset, length, obj', response[0].config.data.offset, response[0].data.values.length, response);

                var fidx = $$get(widget, "options.imageColumn.fidx", -1);
                if (fidx >= 0 && defined(response[0].data.values)) {
                    _.each(response[0].data.values, function (val) {
                        if (val[fidx].text.indexOf("<img") === -1 && val[fidx].data.toString().indexOf("<img") === -1) {
                            val[fidx].text = "<div style='margin:0 auto; text-align:center;'><img src='" + val[fidx].text + "'></div>";
                            val[fidx].data = "<div style='margin:0 auto; text-align:center;'><img src='" + val[fidx].data + "'></div>";
                        }

                    });
                }
                callback(defined(response, '0.data.values') ? response[0].data.values : [], null);

            }).catch(function (response) {
                callback(null, response);
                $$.error(response);
            });
        };
    }

    // Registering dashboard/ widget creation in order to perform drilling
    prism.on("dashboardloaded", function (e, args) {
        _.each(args.dashboard.widgets.toArray(), function (w) {
            onWidgetInitialized(w);
        });

        args.dashboard.on("widgetadded", function (w, args) {
            if (args.widget.type === "pivot" && defined(args, "widget.options.imageColumn")) {
                args.widget.on("processresult", addImagePivot);
                args.widget.refresh();
            }
            else if (args.widget.type === "tablewidget" && defined(args, "widget.options.imageColumn")) {
                args.widget.on("processresult", updateImageTable);
                args.widget.refresh();
            }
        })
    });

    // register widget events upon initialization
    function onWidgetInitialized(widget) {
        // hooking to pivot ready/destroyed events
        if (widget.type === "pivot" && defined(widget, "options.imageColumn")) {
            //	For pivot widgets
            widget.on("processresult", addImagePivot);
        } else if (widget.type === "tablewidget" && defined(widget, "options.imageColumn")) {
            // 	For table widgets
            widget.on("processresult", updateImageTable);

        }
        widget.on("destroyed", onWidgetWasDestroyed);
    }

    // unregistering widget events
    function onWidgetWasDestroyed(widget, args) {
        if (widget.type === "pivot") {
            //	For pivot widget
            widget.off("processresult", addImagePivot);
        } else {
            //	For table widget
            widget.off("processresult", updateImageTable);
        }
        widget.off("destroyed", onWidgetWasDestroyed);
    }

    // create dashboards menu
    prism.on("beforemenu", function (e, args) {
        try {

            //	Define menu caption
            var menuCaption = "Image URL Column";

            //	Hooking to the WidgetEditor Menu
            if (!($(args.ui.target).hasClass('js--btn-widget-menu') || $(args.ui.target).hasClass('wet-menu'))) {
                return;
            }

            //	Has the menu been added already?
            var addMenuItem = true;
            $.each(args.settings.items, function () {
                if (this.caption == menuCaption) {
                    addMenuItem = false;
                }
            });

            if (addMenuItem) {
                //	Save column selection
                function setColumn() {
                    //	Set the image column as an option
                    e.targetScope.widget.options.imageColumn = {fidx: this.fidx, caption: this.caption};
                    //	Update the widget now

                    if (e.targetScope.widget.type === "pivot") {
                        if ((!defined(e.targetScope.widget.options.imageColumn) || e.targetScope.widget.options.imageColumn.fidx < 0))
                            e.targetScope.widget.off("processresult", addImagePivot);
                        else {
                            e.targetScope.widget.off("processresult", addImagePivot);
                            e.targetScope.widget.on("processresult", addImagePivot);
                        }

                    }
                    else {
                        if ((!defined(e.targetScope.widget.options.imageColumn) || e.targetScope.widget.options.imageColumn.fidx < 0))
                            e.targetScope.widget.off("processresult", updateImageTable);
                        else {
                            e.targetScope.widget.off("processresult", updateImageTable);
                            e.targetScope.widget.on("processresult", updateImageTable);
                        }
                    }

                    e.targetScope.widget.refresh();
                }

                //	Look for existing setting
                var myColumn = e.currentScope.widget.options.imageColumn;

                //	Create array of column options
                var columns = [];

                //	Create "disabled" option
                var disabledMenuItem = {
                    caption: "Disable",
                    type: "option",
                    execute: setColumn,
                    checked: (!defined(myColumn) || myColumn.fidx < 0) ? true : false,
                    fidx: -1
                };
                columns.push(disabledMenuItem);

                //	Get the available dimensions
                if (e.currentScope.widgetType == "tablewidget") {

                    //	Working with a table widget
                    var dimensions = $.grep(e.currentScope.widget.metadata.panels, function (a) {
                        return a.name == "columns";
                    })[0];

                    //	Loop through the list, and create sub-menus
                    $.each(dimensions.items, function (i) {
                        //var index = i+1;
                        var menuItem = {
                            caption: this.jaql.title,
                            type: "option",
                            execute: setColumn,
                            checked: (defined(myColumn) && i == myColumn.fidx) ? true : false,
                            fidx: i
                        };
                        columns.push(menuItem);
                    })

                } else {

                    //	Working with a pivot widget
                    var dimensions = $.grep(e.currentScope.widget.metadata.panels, function (a) {
                        return a.name == "rows";
                    })[0];

                    //	Loop through the list, and create sub-menus
                    //	Loop through the list, and create sub-menus
                    $.each(dimensions.items, function () {
                        var menuItem = {
                            caption: this.jaql.title,
                            type: "option",
                            execute: setColumn,
                            checked: (defined(myColumn) && this.field.index == myColumn.fidx) ? true : false,
                            fidx: this.field.index
                        };
                        columns.push(menuItem);
                    })
                }

                //	Add the new menu items
                args.settings.items.push({type: "separator"});
                args.settings.items.push({
                    caption: menuCaption,
                    items: columns
                });
            }

        }
        catch (e) {
            console.log(e);
        }
    });

    //create dashboard links with filters for each pivot cell
    function addImagePivot(widget, query, result, rawResult, reason) {
        try {
            // Clear selected dimension index if it not exist
            if (widget.options.imageColumn.fidx > widget.metadata.panel("rows").items.length - 1) {
                widget.options.imageColumn.fidx = -1;
            }
            //	Get the column index
            var index = widget.options.imageColumn.fidx;
            //	Get the HTML table
            var table = $(query.result.table);
            //	Get the selected cells
            var cells = $('td[fidx=' + index + '][class*=p-dim-member]', table);

            //	Loop through each cell and update the contents
            $.each(cells, function () {
                //	Create the HTML
                if ($(this).text().length > 0) {
                    var imgTag = "<div style='margin:0 auto;'><img src='" + $(this).text() + "'></div>";
                    //	Override the existing cell contents
                    $(this).html(imgTag);
                    //	Add CSS styling to center the images
                    $(this).css("text-align", "center");
                }
            });
            //	Write the new HTML table back to the widget
            query.result.table = table[0].outerHTML;
        }
        catch (e) {
            console.log(e);
        }
    }

    function updateImageTable(widget, args) {
        try {
            //	Get the current widget's html element
            var element = prism.$ngscope.appstate === "dashboard" ? $('widget[widgetid=' + widget.oid + ']') : $('tablewidget');//this.parentElement.parentElement;
            // Clear selected dimension index if it not exist
            if (widget.options.imageColumn.fidx > widget.metadata.panel("columns").items.length - 1) {
                widget.options.imageColumn.fidx = -1;
            }
            //	Get the column index
            var fidx = widget.options.imageColumn.fidx;
            //	Make sure some column is specified
            if (fidx >= 0) {
                _.each(args.result.$$rows, function (row) {
                    if (row[fidx].text.indexOf("<img") === -1 && row[fidx].data.toString().indexOf("<img") === -1) {
                        row[fidx].text = "<div style='margin:0 auto; text-align:center;'><img src='" + row[fidx].text + "'></div>";
                        row[fidx].data = "<div style='margin:0 auto; text-align:center;'><img src='" + row[fidx].data + "'></div>";
                    }

                });
            }
        }
        catch (e) {
            console.log(e);
        }
    }
}]);